REBOL [
  Title: "LIST-VIEW Document and Images Builder"
  Author: ["Henrik Mikael Kristensen"]
  Copyright: "2006 - HMK Design"
  Filename: %doc-images.r
  Version: 0.0.1
  Created: 02-Feb-2006
  Date: 26-Mar-2006
  License: {
    BSD (www.opensource.org/licenses/bsd-license.php)
    Use at your own risk.
  }
  Purpose: {
    Run this file to create documentation and images for LIST-VIEW
  }
  History: []
]

do/args %../demos/build-demos.r 'from-doc-images

do either exists? %../../list-view.r [%../../list-view.r][%list-view.r]
do/args either exists? %../../makedoc2-original.r [
  %../../makedoc2-original.r
][
  %makedoc2-original.r
] 'load-only

save-layout: func [file lo /nohide] [
  image-list/append-row/values to-string file
  ; if window already open, caused by /nohide,
  ; just capture the contents, otherwise open a new one
  if 1 = length? system/view/screen-face/pane [
    view/new either object? lo [lo][layout/tight/offset lo 1600x0]
  ]
  save/png file to image! system/view/screen-face/pane/2
  unless nohide [unview]
  wait 0.001
]

save-doc: func [file [file!]] [
  doc: scan-doc replace read to-file file "$date" to-string now/date
  set [title out] gen-html/options doc [(options)]
  out-file: copy/part to-string file subtract length? to-string file 4
  write to-file join out-file '.html out
  image-list/append-row/values to-string file
  wait 0.001
]

view/new image-lo: layout [
  origin 4 space 2
  h1 "Building Docs and Images"
  image-list: list-view 300x302 with [
    data-columns: [image]
    header-columns: []
  ] edge [size: 1x1]
  across
  btn "Quit" [quit]
  btn "Rebuild" [image-list/clear build]
  btn "Show Doc" [browse %list-view.html]
  btn "Show Demo Page" [browse %../demos/index.html]
  btn "Show History" [browse %list-view-history.html]
]


build: does [
  save-doc %list-view.txt
  save-doc %list-view-history.txt
  save-doc %../demos/index.txt
  save-layout %plain1.png [list-view]

save-layout %demo.png [
  list-view with [
    data-columns: [Name Age Location]
    sort-column: 'name
    data: [
      [Chewbacca 40 Tatooine]
      ["Luke Skywalker" 19 Tatooine]
      ["Qui-Gon Jinn" 48 Coruscant]
      ["Obi-Wan Kenobi" 35 Coruscant]
      ["Palpatine" 60 Coruscant]
      ["Padm Amidala" 20 Naboo]
      ["Darth Vader" 50 "Death Star"]
    ]
  ]
]

save-layout %data-columns.png [
  list-view with [
    data-columns: [Age Height Weight]
  ]
]

save-layout %data-single.png [
  li: list-view with [
    data: ["Eenie" "Meenie" "Miney" "Moe"]
  ]
]

a: layout/tight [
  li: list-view with [
    data: [
      ["Eenie" "Meenie"]
      ["Miney" "Moe"]
    ]
  ]
]

save-layout %data-block.png a

set-face li [["Ernie" "Bernie"]]

save-layout %set-face.png a

clear-face li

save-layout %clear-face.png a

save-layout %data-block-objects.png [
  list-view with [
    data: reduce [
      make object! [name: "Eenie" last-name: "Meenie"]
      make object! [name: "Miney" last-name: "Moe"]
    ]
  ]
]

a: layout/tight [
  li: list-view with [
     default-object: make object! [
       name: "Name not given"
       last-name: "Last name not given"
     ]
   ]
]

li/append-row

save-layout %default-object.png a

save-layout %incolsonly.png [
  list-view with [
    data-columns: [Age Height Weight]
    data: [
      [46 163 56]
      [32 172 56]
    ]
  ]
]

save-layout %colsorted.png [
  list-view 200x100 with [
    data-columns: [a b c d e f]
    viewed-columns: sort/reverse data-columns
  ]
]

save-layout %addcols1.png a: layout/tight [
  li: list-view 200x100 with [
    data-columns: [a b c]
    data: [[1 2 3][4 5 6]]
  ]
]

li/append-column 'd
li/update

save-layout/nohide %addcols2.png a

li/insert-column 2 's
li/update

save-layout/nohide %addcols3.png a

li/remove-column 'c
li/update

save-layout/nohide %addcols4.png a

li/update-column 'a 'y
li/update

save-layout %addcols5.png a

save-layout %manycols.png [
  list-view with [
    data-columns: [a b c d e f g h i j k]
  ]
]


save-layout %incolsoutcols.png [
  list-view with [
    data-columns: [Age Height Weight]
    viewed-columns: [weight age]
    data: [
      [46 163 56]
      [32 172 56]
    ]
  ]
]

save-layout %colsizepixels.png [
  list-view with [
    data-columns: [Age Height Weight]
    viewed-columns: [weight age]
    fit: false
    widths: [75 75]
    data: [
      [46 163 56]
      [32 172 56]
    ]
  ]
]

save-layout %colsizedecimal.png [
  list-view with [
    data-columns: [Age Height Weight]
    viewed-columns: [weight age]
    widths: [0.2 0.8]
    data: [
      [46 163 56]
      [32 172 56]
    ]
  ]
]

save-layout %colsizemixed.png [
  list-view with [
    data-columns: [Age Height Weight]
    viewed-columns: [weight age]
    fit: false
    widths: [75 0.6]
    data: [
      [46 163 56]
      [32 172 56]
    ]
  ]
]

save-layout %colsizetoobig.png [
  list-view with [
    data-columns: [Age Height Weight]
    viewed-columns: [weight age]
    fit: false
    widths: [0.2 1.8]
    data: [
      [46 163 56]
      [32 172 56]
    ]
  ]
]

save-layout %hscroll.png [
  list-view with [
    data-columns: [Age Height Weight]
    viewed-columns: [weight age]
    widths: [0.2 0.8]
    h-scroll: true
    data: [
      [46 163 56]
      [32 172 56]
    ]
  ]
]

save-layout %buddies.png a: layout/tight [
  buddies: list-view 200x100 with [
    data-columns: [Name Status]
    data: [
      ["James" away]
      ["John" here]
      ["Joe" here]
    ]
  ]
]

buddies/change-row ["James" offline]

save-layout %changebuddies.png a

buddies/find-row ["Joe" here]

save-layout %findbuddyrow.png a

buddies/find-row "Joe"

save-layout %findbuddy.png a

buddies/find-row/col "John" 'name

save-layout %findbuddybycol.png a

buddies/change-row/top ["John" away]

save-layout %changebuddiestop1.png a

buddies/find-row/col "Joe" 'name
buddies/change-row/top ["Joe" away]

save-layout %changebuddiestop2.png a

buddies/find-row/col "James" 'name
buddies/change-row/top ["James" here]

save-layout %changebuddiestop3.png a

buddies/change-row/top buddies/find-row/col "Joe" 'name

save-layout %changefindbuddy.png a

buddies/find-row/col "James" 'name
buddies/change-cell 'status 'here

save-layout %changecellfindbuddy.png a

a: layout/tight [
  results: list-view 200x100 with [
    data-columns: [Name Time]
    data: [
      ["Clara" 0:45:41]
      ["Carla" 0:42:47]
      ["James" 0:51:1]
      ["Carrie" 0:47:33]
      ["Carol" 0:43:13]
      ["Jim" 0:44:26]
    ]
  ]
]

results/set-limit 3

save-layout/nohide %limit.png a

results/set-sorting 'time 'asc
results/update

save-layout %rowheight.png [
  list-view with [
    row-height: 40
    data: ["Really" "Big" "Rows"]
  ]
]

save-layout %listunresized.png a: layout/tight [
  li: list-view 200x100
]

a/size: 300x120 ; necessary to show the list correctly
li/resize 300x120
li/update
show a

save-layout %listresized.png a

save-layout %noselect.png a: layout/tight [
  lv: list-view 300x120 with [
    data-columns: [A B C D E F G]
    data: [
      [a b c d e f g]
      [h i j k l m n]
      [o p q r s t u]
      [v w x y z 0 1]
      [2 3 4 5 6 7 8]
    ]
  ]
]

lv/context-list/hilight-cell 1 2

save-layout %firstselectcell.png a

lv/context-list/hilight-cell/shift 4 5

save-layout %lastselectcell.png a

lv/context-list/hilight-cell
  lv/col-idx 'b ; Column B
  pick lv/sort-index 4 ; Fourth Row

save-layout %selecttranslation.png a

lv/context-list/hilight-horizontal/shift
  lv/col-idx 'd
  pick lv/sort-index 3

save-layout %selecthorizontal.png a

lv/context-list/hilight-vertical/shift
  lv/col-idx 'd
  pick lv/sort-index 3

save-layout %selectvertical.png a

lv/context-list/hilight-cell 2 2
lv/context-list/hilight-cell/control 4 4
lv/context-list/hilight-cell/control 6 3

save-layout %selectcontrol.png a

save-layout %rowspacing.png [
  list-view with [
    data-columns: [Spaced Out Columns And Rows]
    spacing: 0x1
  ]
]

save-layout %colspacing.png [
  list-view with [
    data-columns: [Spaced Out Columns And Rows]
    spacing: 1x0
  ]
]

save-layout %rowcolspacing.png [
  list-view with [
    data-columns: [Spaced Out Columns And Rows]
    spacing: 1x1
  ]
]

save-layout %spacingcolor.png [
  list-view with [
    data-columns: [Spaced Out Columns And Rows]
    spacing: 2x2
    spacing-color: 0.100.150
  ]
]

save-layout %standardfont.png [
  list-view 300x100 with [
    data: [["Eenie" "Meenie"]["Miney" "Moe"]]
    standard-font: make standard-font [name: "tahoma"]
  ]
]

save-layout %standardpara.png [
  list-view 300x100 with [
    data: [["Eenie" "Meenie"]["Miney" "Moe"]]
    standard-para: make standard-para []
  ]
]

save-layout %standardheaderfont.png [
  list-view 300x100 with [
    data: [["Eenie" "Meenie"]["Miney" "Moe"]]
    standard-header-font: make standard-header-font [
      name: "tahoma"
      size: 11
      style: 'bold
      shadow: none
      align: 'center
    ]
  ]
]

save-layout %standardheaderpara.png [
  list-view 300x100 with [
    data: [["Eenie" "Meenie"]["Miney" "Moe"]]
    standard-header-para: make standard-header-para [
      origin: 20x2
    ]
  ]
]

save-layout %limitsorted.png a

a: layout/tight [
  li: list-view 300x120 with [
    data: [
      ["123" "456" "789"]
      ["Eenie" "Meenie" "Miney"]
      ["Red" "Green" "Blue"]
      ["Yellow" "Black" "Brown"]
    ]
  ]
]

li/context-list/hilight-cell 2 2
li/update

save-layout/nohide %getrangesingle.png a

li/context-list/hilight-cell 3 1
li/context-list/hilight-cell/control 2 2
li/context-list/hilight-cell/control 1 3
li/update

save-layout/nohide %getrangearbitrary.png a

li/context-list/hilight-row 1
li/update

save-layout/nohide %getrangerow.png a

li/context-list/hilight-cell 2 1
li/context-list/hilight-cell/shift 3 3
li/update

save-layout/nohide %getrangesquare.png a

li/context-list/hilight-column 3
li/update

save-layout %getrangecolumn.png a

unset [a li]

save-layout %truncatefalse.png a: layout/tight [
  li: list-view 150x100 with [
    data: ["A really really long sentence"]
    truncate: false
  ]
]

li/truncate: true
li/update

save-layout %truncatetrue.png a

save-layout %truncaterowaction.png [
  li: list-view 150x100 with [
    data: ["A really really long sentence"]
    truncate: true
    row-action: [
       cell/text: reverse cell/text
    ]
  ]
]

save-layout %images.png [
  list-view 200x300 with [
    data-columns: [Images]
    data: reduce [help.gif info.gif logo.gif stop.gif]
    row-height: 50
  ]
]

save-layout %imagestext.png [
  list-view 300x300 with [
    data-columns: [Image Description]
    data: compose/deep [
      [(help.gif) "Help Image"]
      [(info.gif) "Info Image"]
      [(logo.gif) "REBOL Logo"]
      [(stop.gif) "Stop Image"]
    ]
    row-height: 50
  ]
]

warning: load %warning.png
fatal: load %fatal.png

save-layout %someimages.png [
  list-view 150x200 with [
    data-columns: [Action Image]
    header-columns: ["Vehicle Startup"]
    widths: [100 20]
    resize-column: 'action
    data: compose/deep [
      ["System Check" (none)]
      ["Ignition" (none)]
      ["Engine" (none)]
      ["Tires" (none)]
      ["Windows" (none)]
      ["Oil Pressure" (warning)]
      ["Brakes" (none)]
      ["Steering" (warning)]
      ["Fuel" (fatal)]
    ]
  ]
]

save-layout %filternormal.png a: layout/tight [
  lv: list-view 200x80 with [
    data: [
      ["a" "b"]
      ["c" "a"]
      ["f" "g"]
    ]
  ]
]

do-filter: func [string][
  lv/filter-string: string
  lv/update
]

do-filter "a"
save-layout/nohide %filter-a.png a

do-filter "c"
save-layout/nohide %filter-c.png a

do-filter "REBOL"
save-layout/nohide %filter-rebol.png a

do-filter ""
lv/editable?: true
lv/first-cnt
lv/show-edit
save-layout %inline.png a

a: layout/tight [
  lv: list-view 500x200 with [
    data-columns: [Firstname Lastname Age Color Height Gender Occupation]
    data: [
      ["James"   "Kirk"      36 red    175 male   "Captain"]
      ["Jim"     "Belushi"   25 blue   182 male   "Actor"]
      ["Jill"    "McGill"    17 blue   171 female "Unknown"]
      ["Jane"    "Doe"       35 green  167 female "Unknown"]
      ["Joe"     "DiMaggio"  46 red    167 male   "Player"]
      ["John"    "Rambo"     42 blue   177 male   "Soldier"]
      ["Jamie"   "Oliver"    21 yellow 177 male   "Cook"]
      ["Judy"    "Dench"     21 yellow 159 female "Actress"]
      ["Jack"    "Nicholson" 41 blue   171 male   "Actor"]
      ["Jack"    "Lemmon"    55 red    169 male   "Actor"]
      ["Jack"    "Bauer"     34 red    178 male   "Agent"]
      ["Henry"   "Ford"      56 yellow 183 male   "Boss"]
      ["Audrey"  "Hepburn"   23 blue   168 female "Actress"]
      ["Ford"    "Fairlane"  33 green  186 male   "Detective"]
      ["Walther" "Matthau"   67 green  191 male   "Actor"]
    ]
  ]
]

save-layout %nofilterspec.png a

lv/set-filter-spec 'color "red" []

save-layout %filterspeccolor.png a

lv/set-filter-spec 'color "blue" []

save-layout %filterspeccolor2.png a

lv/set-filter-spec 'gender "male" []

save-layout %filterspecgender.png a

lv/set-filter-spec/only 'gender 'male []

save-layout %filterspecgenderonly.png a

lv/remove-filter-spec 'color

save-layout %filterspecremove.png a

lv/set-filter-spec/only 'gender 'male [gender]

save-layout %filterspecgendercolumn.png a

lv/set-filter-spec/only/not 'color 'red [color]

save-layout %filterinversed.png a

lv/set-filter-spec 'name "ford" [firstname]

save-layout %filterspecfirstname.png a

lv/set-filter-spec 'name "ford" [lastname]

save-layout %filterspeclastname.png a

lv/set-filter-spec 'name "ford" [firstname lastname]

save-layout %filterspecnamecolumns.png a

lv/reset-filter-specs

save-layout %filterspecreset.png a

a: layout/tight [
  li: list-view 400x100 with [
    data-columns: [a b c]
    readonly-columns: [b]
  ]
]

li/append-row/values [
  "Edit this column"
  "You can't edit this one!"
  "Edit this column"
]

save-layout/nohide %readonly1.png a

li/show-edit

save-layout %readonly2.png a

a: layout/tight [
  li: list-view 300x100 with [
    editable?: true
    data-columns: [age weight height]
    edit-action: [
      set-face age first parse get-cell 'age none
      set-face weight first parse get-cell 'weight none
      set-face height first parse get-cell 'height none
      true
    ]
  ]
]

li/append-row/values ["58 years" "73 kg" "171 cm"]

save-layout/nohide %editaction1.png a

li/show-edit

save-layout/nohide %editaction2.png a

save-layout/nohide %tabeditaction.png a

li/pre-submit-edit-action: [
  age/text:    join age/text " years"
  weight/text: join weight/text " kg"
  height/text: join height/text " cm"
]

li/pre-submit-edit-func
li/hide-edit
li/update

save-layout %finisheditaction.png a

save-layout %withhdrcols.png [
  list-view 200x100 with [
    data-columns: [a b c]
    viewed-columns: [b a]
    header-columns: ["B" "A"]
  ]
]

save-layout %withouthdrcols.png [
  list-view 200x100 with [
    data-columns: [a b c]
    viewed-columns: [b a]
  ]
]

save-layout %withemptyhdrcols.png [
  list-view 200x100 with [
    data-columns: [a b c]
    viewed-columns: [b a]
    header-columns: []
  ]
]

a: layout/tight [
  sort-list: list-view 300x120 with [
    data-columns: [Name Age Weight Height]
    data: [
      ["James" 36 72 160]
      ["Jim" 12 52 180]
      ["Joe" 24 88 192]
    ]
  ]
]

sort-list/set-sorting 'weight 'asc

save-layout %sortsettings.png a

sort-list/append-row/values ["Jake" 56 66 175]

save-layout %appendsort.png a

save-layout %allowsorting.png [
  list-view 160x100 with [
    allow-sorting: false
  ]
]

save-layout %conditionexample.png [
  list-view 100x160 with [
    data-columns: [name status role]
    viewed-columns: [name]
    header-columns: ["Buddy List"]
    data: [
      ["James" here user]
      ["Jake" away admin]
      ["John" offline user]
      ["Jim" here user]
      ["Joe" here guest]
    ]
    conditions: [
      [[role = 'admin] font [style: 'bold]]
      [[status = 'away] font [color: 150.150.150]]
    ]
  ]
]

save-layout %conditioncolumnexample.png [
  list-view 200x160 with [
    data-columns: [Name Status Role Last-seen]
    viewed-columns: [name last-seen]
    data: [
      ["James" here user 0:18]
      ["Jake" away admin 15:35]
      ["John" offline user 14:10]
      ["Jim" here user 0:56]
      ["Joe" here guest 0:45]
    ]
    conditions: [
      [[role = 'admin] font [style: 'bold] [name]]
      [[status = 'away] font [color: gray]]
      [[last-seen < 12:00] font [color: blue] [last-seen]]
    ]
  ]
]

save-layout %smallscroller.png [
  list-view 150x100 with [
    scroller-width: 15
  ]
]

save-layout %bigscroller.png [
  list-view 150x100 with [
    scroller-width: 30
  ]
]

save-layout %completecustom.png [
  y: list-view 270x300 with [
    data: [
      [Chewbacca 40 Tatooine]
      ["Luke Skywalker" 19 Tatooine]
      ["Qui-Gon Jinn" 48 Coruscant]
      ["Obi-Wan Kenobi" 35 Coruscant]
      [Palpatine 60 Coruscant]
      ["Padm Amidala" 20 Naboo]
      ["Darth Vader" 50 "Death Star"]
    ]
    data-columns: [Name Age Location]
    header-columns: ["Star Wars Characters"]
    sort-column: 'age
    row-face: [
      across space 0
      list-text 200x20
      list-text 50x20 reblue font-size 14 bold right return
      list-text 250x15 gray right font-size 10
    ]
  ]
]

save-layout %fonts.png [
  list-view with [
    data-columns: [Name Level Score]
    data: [
      ["Joe" 12 5645.6]
      ["James" 6 4472.2]
      ["Jimmy" 11 5631.7]
    ]
    fonts: [
      [style: 'bold]
      [align: 'center]
      [align: 'right]
    ]
  ]
]

save-layout %fontsobj.png [
  list-view 100x160 with [
    data-columns: [Name]
    data: [
      ["Joe"]
      ["James"]
      ["Jimmy"]
    ]
    fonts: [
      make object! [size: 14 style: 'bold color: red align: 'center]
    ]
  ]
]

save-layout %widths1.png [
  list-view with [
    data-columns: [Name Age Location]
    widths: [60 40 120]
    data: [
      ["Eenie" "42" "Uranus"]
      ["Miney" "31" "Pluto"]
    ]
    resize-column: 'name
  ]
]

save-layout %widths2.png [
  list-view with [
    data-columns: [Name Age Location]
    widths: [60 40 120]
    data: [
      ["Eenie" "42" "Uranus"]
      ["Miney" "31" "Pluto"]
    ]
    resize-column: 'location
  ]
]

save-layout %widths3.png [
  list-view with [
    data-columns: [Name Age Location]
    widths: [0.5 0.3 0.2]
    data: [
      ["Eenie" "42" "Uranus"]
      ["Miney" "31" "Pluto"]
    ]
  ]
]

save-layout %widths4.png [
  list-view with [
    data-columns: [Name Age]
    widths: [0.5 200]
    data: [
      ["Eenie" "42"]
      ["Miney" "31"]
    ]
  ]
]

a: layout/tight [
  li: list-view 150x100 with [
    data: [["List data"]]
  ]
]

li/first-cnt
li/focus-list

save-layout %focusedlist.png a

li/unfocus-list

save-layout %unfocusedlist.png a

save-layout %listfaces.png [
    space 1 origin 1
    backdrop black
    box "HDR" 200x20 180.180.180 across
    box 180.180.180 179x179 "LST"
    box 180.180.180 20x179 "SCR" return
    box 180.180.180 179x20 "HSCR"
    box black 20x20
  ]
]

save-layout %rowaction1.png [
  list-view 200x120 with [
    data-columns: [Core Temperature]
    data: [
      ["Reactor Core 1" 1400]
      ["Reactor Core 2" 3400]
      ["Reactor Core 3" 7600]
      ["Reactor Core 4" 4500]
    ]
    row-action: [
      cell/font/color: either temperature > 5000 [red][black]
    ]
  ]
]

save-layout %rowaction2.png [
  list-view 120x120 with [
    lock-list: true
    data-columns: [name age]
    data: [
      ["Jack" 37]
      ["James" 26]
      ["Jim" 65]
      ["Joan" 45]
    ]
    row-action: [
      cell/font/color: either lock-list [gray][black]
    ]
  ]
]

save-layout %rowaction3.png [
  list-view 100x160 with [
    data-columns: [name status role]
    viewed-columns: [name]
    data: [
      ["James" here user]
      ["Jake" away admin]
      ["John" offline user]
      ["Jim" here user]
      ["Joe" here guest]
    ]
    row-action: [
      cell/font/style: none
      cell/font/color: black
      if role = 'admin [cell/font/style: 'bold]
      if status = 'away [cell/font/color: gray]
    ]
  ]
]

save-layout %rowaction4.png [
  list-view 120x160 with [
    data-columns: [Name Status Role]
    viewed-columns: [name status]
    header-columns: ["Buddies"]
    data: [
      ["James" here user]
      ["Jake" away admin]
      ["John" offline user]
      ["Jim" here user]
      ["Joe" here guest]
    ]
    row-action: [
      cells/status/color: switch/default status [
        here    [0.200.0]
        away    [red]
        offline [gray]
      ][black]
    ]
  ]
]

save-layout %rowaction5.png [
  list-view 120x160 with [
    data-columns: [Name Status Role]
    viewed-columns: [name status]
    header-columns: ["Buddies"]
    widths: [80 20]
    data: [
      ["James" here user]
      ["Jake" away admin]
      ["John" offline user]
      ["Jim" here user]
      ["Joe" here guest]
    ]
    row-action: [
      cells/name/font/style: none
      if role = 'admin [cells/name/font/style: 'bold]
      cells/status/text: none
      cells/status/effect: switch/default status [
        here    [[draw [pen black fill-pen 0.200.0 translate 5x5 circle 5x5]]]
        away    [[draw [pen black fill-pen red translate 5x5 circle 5x5]]]
        offline [[draw [pen black fill-pen gray translate 5x5 circle 5x5]]]
      ][none]
    ]
  ]
]

save-layout %rowaction6.png [
  list-view 120x160 with [
    data-columns: [Name Status Role]
    viewed-columns: [name status]
    header-columns: ["Buddies"]
    widths: [80 20]
    data: [
      ["James" here user]
      ["Jake" away admin]
      ["John" offline user]
      ["Jim" here user]
      ["Joe" here guest]
    ]
    row-action: [
      switch column [
        name [
          cells/name/font/style: none
          if role = 'admin [cells/name/font/style: 'bold]
        ]
        status [
          cells/status/text: none
          cells/status/effect: switch/default status [
            here    [[draw [pen black fill-pen 0.200.0 translate 5x5 circle 5x5]]]
            away    [[draw [pen black fill-pen red translate 5x5 circle 5x5]]]
            offline [[draw [pen black fill-pen gray translate 5x5 circle 5x5]]]
          ][none]
        ]
      ]
    ]
  ]
]

a: layout/tight [
  li: list-view 120x160 with [
    data-columns: [Name Status Role]
    viewed-columns: [name status]
    header-columns: ["Buddies"]
    widths: [80 20]
    data: [
      ["James" here user]
      ["Jake" away admin]
      ["John" offline user]
      ["Jim" here user]
      ["Joe" here guest]
    ]
    row-action: [
      cell/effect: copy either sel-cnt = row [
        [gradient 0x1 0.120.240 0.80.120]
      ][[]]
      switch column [
        name [
          cells/name/font/style: none
          if role = 'admin [cells/name/font/style: 'bold]
        ]
        status [
          cells/status/text: none
          append cells/status/effect switch/default status [
            here    [[draw [pen black fill-pen 0.200.0 translate 5x5 circle 5x5]]]
            away    [[draw [pen black fill-pen red translate 5x5 circle 5x5]]]
            offline [[draw [pen black fill-pen gray translate 5x5 circle 5x5]]]
          ][none]
        ]
      ]
    ]
    sel-cnt: 2
  ]
]

li/find-row/col 'away 'status

save-layout %rowaction7.png a

draw-settings: [
  anti-alias off line-width 1 line-join round pen 100.100.100
  translate -5x0
]

save-layout %rowaction8.png layout/tight [
  lv: list-view 200x200 with [
    data-columns: [Section Page Title]
    viewed-columns: [Section Page Title]
    header-columns: ["" "Page" "Title"]
    resize-column: 'title
    widths: [20 40 140]
    group-rows-by: 'Section ; set this for the column you want to group by
    row-action: [
      if column = 'section [
        cell/text: none
        ; group-row-type responds to GROUP-ROWS-BY
        cell/effect: compose/deep switch group-row-type [
          start [
            [draw [(draw-settings) line 12x20 12x10 18x10]]
          ]
          between [
            [draw [(draw-settings) line 12x0 12x20]]
          ]
          end [
            [draw [(draw-settings) line 12x0 12x10 18x10]]
          ]
          single [
            [draw [(draw-settings) line 12x10 18x10]]
          ]
        ]
      ]
    ]
    data: [
      [Start 1.1 "Front page"]
      [Start 1.2 "Table of Contents"]
      [Chapter1 2.1 "Chapter 1"]
      [Chapter1 2.2 "Chapter 1"]
      [Chapter1 2.3 "Chapter 1"]
      [Chapter2 3.1 "Chapter 2"]
      [Chapter2 3.2 "Chapter 2"]
      [Chapter2 3.3 "Chapter 2"]
      [End 4.1 "Back page"]
    ]
  ]
]

build
do-events
quit
